/**
 * Method for running a single process
 * @type FetchResult Result type for the valve
 * @type SubqueueKeyType Subqueue unique identifier type
 * @param subqueue Unique key of the subqueue being run
 */
export declare type FetcherProcess<FetchResult, SubqueueKeyType = string | number | symbol> = (subqueue?: SubqueueKeyType) => Promise<FetchResult>;
/**
 * Method for running a batch fetch process
 * @type FetchResult Result type for the valve
 * @type SubqueueKeyType Subqueue unique identifier type
 * @param subqueues Unique keys of the subqueues being run
 * @param earlyWrite Mechanism for unblocking subqueues as the data is availiable
 */
export declare type BatchFetcherProcess<FetchResult, SubqueueKeyType = string | number | symbol> = (subqueues: SubqueueKeyType[], earlyWrite: (subqueue: SubqueueKeyType, result: FetchResult | Error) => void) => Promise<Array<FetchResult | Error> | Map<SubqueueKeyType, FetchResult | Error> | void>;
/**
 * Configurable parameters when creating a new instance of BurstValve
 */
export interface BurstValveParams<DrainResult, SubqueueKeyType> {
    /**
     * Display name for the valve (useful for debugging exceptions)
     */
    displayName?: string;
    /**
     * Fetcher process for single concurrency process running
     */
    fetch?: FetcherProcess<DrainResult, SubqueueKeyType>;
    /**
     * Fetcher process for single concurrency on a list
     * of unique identifiers
     */
    batch?: BatchFetcherProcess<DrainResult, SubqueueKeyType>;
}
/**
 * Concurrent queue for a single (or batch) asynchronous action
 * @type DrainResult Result type when queue is drained
 * @type SubqueueKeyType Error type when queue is drained with an error
 */
export declare class BurstValve<DrainResult, SubqueueKeyType = string | number | symbol> {
    /**
     * Display name for the valve
     */
    readonly displayName: string;
    /**
     * Fetcher for single concurrent running of a function
     */
    private readonly fetcher?;
    /**
     * Fetcher for batch of unique identifiers to run once
     */
    private readonly batchFetcher?;
    /**
     * Queue of promise callbacks
     */
    private queue?;
    /**
     * Keyed subqueues of promise callbacks
     */
    private subqueues;
    /**
     * Creates an instance of BurstValve with a custom fetcher
     * @param fetcher Fetcher process for single concurrency process running
     */
    constructor(fetcher: FetcherProcess<DrainResult, SubqueueKeyType>);
    /**
     * Creates an instance of BurstValve with a custom display name and fetcher
     * @param displayName Name for the valve
     * @param fetcher Fetcher process for single concurrency process running
     */
    constructor(displayName: string, fetcher: FetcherProcess<DrainResult, SubqueueKeyType>);
    /**
     * Creates an instance of BurstValve with configurable parameters
     * @param config Burst value configuration
     */
    constructor(config: BurstValveParams<DrainResult, SubqueueKeyType>);
    /**
     * Determines if queue (or subqueue) has an active action being taken
     * @param subqueue Unique identifier of the subqueue to check activity.
     */
    isActive(subqueue?: SubqueueKeyType): boolean;
    /**
     * Leverages the current valve to only have a single running process of a function
     * @param subqueue Unique identifier of the subqueue to fetch data for
     */
    fetch(subqueue?: SubqueueKeyType): Promise<DrainResult>;
    /**
     * Batches fetching of unique identifiers into a single process, waiting
     * for existing queues if they already exist
     *
     * @param subqueues List of unique identifiers to fetch at once
     */
    batch(subqueues: SubqueueKeyType[]): Promise<Array<DrainResult | Error>>;
    /**
     * Flushes the queue specified with the result passed
     * @param subqueue Unique identifier tied to the fetch process
     * @param result Successful/Failed result of the fetch process
     */
    private flushResult;
}
//# sourceMappingURL=index.d.ts.map